@@ -21,6 +21,10 @@ extern pr int __mp_dummyvar;        /* make multiprefix work correctly */
 #define mpand(a,b) (__mp_dummyvar = mpand((a),b))
 #define mpor(a,b) (__mp_dummyvar = mpor((a),b))
                                     /* bug fix to guarantee side effect */
+extern void syncadd (int *, int);
+extern void syncmax (int *, int);
+extern void syncand (int *, int);
+extern void syncor (int *, int);
 
 extern sh const int __STARTED_PROCS__; /* number of started processors */
                                        /* generally unknown at compile time */
@@ -39,17 +43,18 @@ extern async void *alloc( unsigned int );
 /* allocate permanent shared heap space:
    alloc for new space, shmalloc for reuse */
 
-extern async void shfree( void * );     /*dummy permanent shared heap free() */
+extern async void shfree( void * );     /*dummy permanent shared heap free()*/
 extern async int pravail( void );     /* returns #free priv. memory words */
 extern async int shavail( void );     /* returns #free shar. memory words */
 
 extern void exit( int );
-extern int getct( void );                /* get global time counter, 1 == 4msec */
+extern unsigned int getct( void );   /* get global time counter, 1cc=4msec */
 
 extern sync int groupsize( void );
 extern async int async_groupsize( void );   /* same function, but other prototype */
                                             /* required for consistency reasons */
 extern sync int parentgroupsize( void );
+extern sync int _groupsize( unsigned int ); /*new 970901*/
 
 
 /* =================  L O C K S :  ======================================== */
@@ -146,14 +151,14 @@ typedef struct {             /* reader-writer-locks */
 
 #define RW_READ 0
 #define RW_WRITE 1
-   
+
 /* the following routines are implemented in forklib2.asm */
 
 #define simple_lock_init(slock) slock=0
 extern void simple_lockup( simple_lock * );
 #define simple_unlock(slock) *(slock)=0
 
-#define fair_lock_init(flock) flock={0,0}
+extern void fair_lock_init(fair_lock *lock);
 extern void fair_lockup( fair_lock * );
 extern int fair_unlock( fair_lock * );
 
