@@ -10,6 +10,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define DEBUG 0
+
+#ifdef FTELL_EXPLICIT
+sh int _filepos[_NFILE];
+#endif
 
 /*-----------output routines----------------------------------------*/
 
@@ -161,6 +166,7 @@ char stdin_linebuf[stdin_linebufsize];
 static int stdin_linecurrent = 0;
 char stdinbuffer[STDINBUFFERSIZE];  /* stack of unmygetchar'd chars*/
 
+
 int mygetchar( char *stdin_buffer )   /*only for STANDARD INPUT, stdin */
 {
    char x;
@@ -341,9 +347,15 @@ int _scanf(
 		  zeroin:
 		    p = ((char *) strchr((char *)digits,toupper(c)));
 		}
+#if DEBUG
+                printf("read n=%d, NEG=%d\n", n, neg);
+#endif
 	      savnum:
 		if (store) {
-		    p = ((char *) *args);
+		    p = ((char *) *(args));
+#if DEBUG
+                printf("store to %d\n", p);
+#endif
 		    if (neg == TRUE)
 			n = -n;
 		    if (lval)
@@ -355,7 +367,7 @@ int _scanf(
 		    ++cnt;
 		}
 		goto breakouterswitch;
-              }		
+              }
 #if FLOATS
 	      /*case 'e':	/* float */ /*case 'f': case 'g':*/
               if (hilf=='e'||hilf=='f'||hilf=='g') {
@@ -388,7 +400,7 @@ int _scanf(
 		    goto done;
 		if (store) {
 		    fx = (*fbuf == '\0') ? 0.0 : atof(fbuf);
-		    p = (char *) *args;
+		    p = (char *) *(args);
 		    if (lval)    *((double *) p) = fx;
 		    else         *((float *) p) = (float)fx;
 		    ++cnt;
@@ -399,7 +411,7 @@ int _scanf(
 	      /*case 'n':*/
               if (hilf=='n') {
 		if (store) {
-		  p = (char *) *args;
+		  p = (char *) *(args);
 		  *((int *) p) = cnt;
 		}
 		goto breakouterswitch;
@@ -456,7 +468,7 @@ int _scanf(
 		skip();
 	      strproc:
 		/* process string */
-		p = ((char *) *args);
+		p = ((char *) *(args));
 		
 		/* if the 1st char fails, match fails */
 		if (width) {
@@ -513,6 +525,9 @@ int _scanf(
 	
 	if (store) {
 	    args++; 
+#if DEBUG
+            printf("incr. args = %d\n", args);
+#endif
         }
 	if (!*++fmt)
 	    break; /*while*/
@@ -531,14 +546,14 @@ int sgetc( char *s )
 {
 	char c;
 
-	c = *(s++);
+	c = *((*(char **)s)++);
 	return((c == '\0') ? EOF : c);
 }
 
 int sungetc( int c, char *s )
 {
   if(c == EOF)   c = '\0';
-  return (*(--s) = c);
+  return (*(--(*(char **)s)) = c);
 }
 
 int sscanf( char *buf, const char *fmt, ... )
@@ -546,7 +561,7 @@ int sscanf( char *buf, const char *fmt, ... )
   va_list argp;
 
   va_start(argp, fmt);
-  return(_scanf((char *) &buf, sgetc, sungetc, (char *)fmt, (char **)argp)); /* &buf */
+  return(_scanf((char *) &buf, sgetc, sungetc, (char *)fmt, (char **)argp+1)); /* &buf */
 }
 
 int fscanf(FILE *fp, const char *fmt, ...)
@@ -555,8 +570,10 @@ int fscanf(FILE *fp, const char *fmt, ...)
 
 	va_start(argp, fmt);
 	return(_scanf((char *)fp, (int (*)(char *))fgetc,
-                      (int (*)(int,char *))ungetc, (char *)fmt, (char **)argp));
+                      (int (*)(int,char *))ungetc, (char *)fmt,
                     /* ^^^^^^ fool the C compiler. CWK */
+                      (char **)argp + 1));
+                               /*   ^^^ notw. aus unerfindlichem Grund 9708 */
 }
 
 int scanf(const char *fmt, ...)
@@ -696,7 +713,7 @@ int fgetc ( FILE *fp )
     int k = read( fp->fd, &c, 1 );
     if (k==0) { fp->flag |= _IOEOF; return EOF; }
  }
-#if 0
+#if DEBUG
  printf("fgetc: %c\n", c);
 #endif
  return c;
@@ -706,7 +723,7 @@ int fgetc ( FILE *fp )
 int ungetc( int x, FILE *fp )
 {
    assert(fp);
-#if 0
+#if DEBUG
    printf("ungetc %c\n", x);
 #endif
    if (x==EOF)  x = (int) '\0';
@@ -770,3 +787,49 @@ char *gets ( char *s )
  }
  return ps;
 }
+
+
+/* direct file access 970811 */
+
+int fseek( FILE *stream, long int offset, int whence )
+{
+ int fd = stream->fd;
+ int ret;
+ 
+ if ( fd > _NFILE ) return -1;   /*error: out of range*/
+ if ( fd < 0 ) return -2;         /*error: out of range*/
+
+ if (whence==SEEK_CUR) { /*first look at buffer */
+      offset = offset - stream->current;
+#ifdef FTELL_EXPLICIT
+      _filepos[fd] += offset;
+#endif
+ }
+ else if (whence == SEEK_SET) {
+    if (offset <0) return 2;  /*error*/
+#ifdef FTELL_EXPLICIT
+    _filepos[fd] = offset;
+#endif
+    stream->current = 0; /*clear buffer:*/
+ } 
+ else if (whence == SEEK_END) {
+    if (offset >0 && (stream->mode & O_RDONLY) ) return 3;  /*error*/
+    assert(0);     /* don't know how to find the filepos of EOF */
+    /* _filepos[fd] = ? */
+ }
+ else return 1;   /*error: whence out of range*/
+
+ ret = lseek( fd, offset, whence );
+ printf("lseek=%d: fd=%d offset=%d whence=%d\n", ret,fd, offset, whence);
+ return ret;
+}
+
+
+long ftell( FILE *stream )
+{
+ int fd = stream->fd;
+ return lseek( fd, 0, SEEK_CUR );
+#ifdef FTELL_EXPLICIT
+ return _filepos[fd]; 
+#endif
+}
