@@ -524,6 +524,8 @@ Node listnodes(tp, tlab, flab) Tree tp; {
       p = node(tp->op, l, r, 0);
       p->synchron = tp->synchron;      /*durchreichen CWK*/
       p->maybe_priv = tp->maybe_pr;    /*CWK*/
+      trash(0);   /*970708 to care about side effects*/
+      list(p);   /*970708 to care about side effects*/
       break;
    case RET:
    case SRET:  /*CWK*/
@@ -616,6 +618,7 @@ Node listnodes(tp, tlab, flab) Tree tp; {
    case SPLIT: case NEXTCASE: case LGRPIF:
    case ENGRP: case EXGRP: case XPROC: case RETFLUSH:
    case ENLGRP: case EXLGRP: case SIZE: case IBP: case FLUSHFRAMES:
+   case BEGSEQ: case ENDSEQ:
       p = newnode(tp->op, 0, 0, 0);
       p->synchron = tp->synchron;      /*durchreichen CWK*/
       break;
@@ -633,7 +636,6 @@ Node listnodes(tp, tlab, flab) Tree tp; {
 /* jump - return tree for a jump to lab */
 Node jump(lab) {
    Symbol p = findlabel(lab);
-
    p->ref++;
    return newnode(JUMP+V, node(ADDRG+P, 0, 0, p), 0, 0);
 }
