@@ -1,4 +1,4 @@
-/* C compiler: VAX subset code generator */
+/* Fork95 code generator, made from lcc1.9's VAX subset code generator */
 
 #include "c.h"
 
@@ -147,7 +147,7 @@ void defconst(int ty, Value v) {
    case D: {
       char buf[MAXLINE];
       /*sprintf(buf, ".double 0d%.18e\n", v.d);*/
-      sprintf(buf, ".double 0e%.18e\n", v.d);   /*CWK*/
+      sprintf(buf, ".float 0e%.18e\n", (float)v.d);   /*CWK 970716*/
       outs(buf);
       break;
       }
@@ -1095,6 +1095,28 @@ CWK 960812   SHADOW wird ganz eliminiert.
             }
          }
          break;
+      case BEGSEQV:                /*CWK 970310*/
+         force_modulo_0;
+         /*synchron. Bestimme leader: Proz. mit kleinster MPADD-priority*/
+         print("getlo\t -1,r31   \t /*BEGSEQV%d*/\n",Pcodepar);
+         print("syncadd\t r31,gps,1\t /*BEGSEQV*/\n");
+         print("getlo\t 1,r31   \t /*BEGSEQV*/\n");
+         print("mpadd\t gps,1,r31\t /*BEGSEQV*/\n");
+         print("nop\t          \t /*BEGSEQV*/\n");
+         print("add\t r31,R0,r31\t /*BEGSEQV*/\n");
+         print("bgt\t ENDSEQ%d\t /*BEGSEQV*/\n", Pcodepar);
+         /* jetzt koennte noch ein shared Gruppenframe fuer 1 Prozessor
+          * aufgebaut werden, um barriers im seq-Rumpf zu erlauben !
+          * das macht aber bei 1 Prozessor wenig Sinn.
+          * Ein privater Frame waere dann auch noetig, damit flushframes
+          * ordentlich arbeitet.*/
+         break;
+      case ENDSEQV:                /*CWK 970310*/
+         /*asynchron. Label und barrier ->weiter synchron in parallel*/
+         /* und genau hier muesste dieser Frame wieder abgeraeumt werden. */
+         print("ENDSEQ%d: bsrg\t spp,forklib_sync /*ENDSEQV%d*/\n",
+                Pcodepar, Pcodepar);
+         break;
       default: assert(0);
       }
    }
@@ -1794,6 +1816,24 @@ CWK 960812
  }
 }
 
+/* begseq(): enter seq body, mask out all but one processor 970310 */
+void begseq( int labelnr ) 
+{
+   Tree e = tree(BEGSEQ+V, voidtype, 0, 0);
+   code(Pcode);
+   codelist->u.node = listnodes( e, 0, 0 );
+   codelist->par = labelnr;
+}
+
+/* endseq(): exit seq body: label, barrier   970310 */
+void endseq( int labelnr ) 
+{
+   Tree e = tree(ENDSEQ+V, voidtype, 0, 0);
+   code(Pcode);
+   codelist->u.node = listnodes( e, 0, 0 );
+   codelist->par = labelnr;
+}
+
 
 /* ================= STAB - Funktionen    CWK 940921 ================== */
 /* adaptiert aus lcc 3.1, files sparc.c und symbolic.c */
