@@ -2,6 +2,8 @@
 
 #include "c.h"
 int isinsyncmode = 0;   /* global flag to pass mode of syncness to function calls CWK*/
+int isinsideastartbody = 0; /* global flag to know whether a start occured*/
+static int SEQcounter = 0;  /*counts SEQ statements, used to generate labels*/
 
 #define SWITCHSIZE 512      /* initial size of switch arrays */
 
@@ -419,10 +421,10 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    if (kind[t] == ID) {
       e1 = expr(';');
       if (may_dep_on_pr_var( e1 ))  /*CWK*/
-        error("fork statement: number of groups must not depend on private var!\n");
+        error("fork: number of groups must not depend on private var!\n");
    }
    else {
-      error("fork statement: number of groups required!\n");
+      error("fork: number of groups required!\n");
       expect(';');
    }
    if (e1) {
@@ -449,13 +451,18 @@ static void forkstmt(lab, swp, lev, bframes, cframes, rframes) struct swtch *swp
    walk(e2, 0, 0);
    definept(0);
    engrp();
-   if (kind[t] == ID)
+   if (kind[t] == ID) {
       e3 = texpr(expr0, ')');
+      walk(e3, 0, 0);
+   }
+   else if (t==')') {
+      warning("fork: no expression renaming $!\n");
+      t = gettok();
+   }
    else {
-      error("fork statement: new proc number expression for $ expected!\n");
+      error("fork: expression renaming $ expected!\n");
       test(')',follow);
    }
-   walk(e3, 0, 0);
    definept(0);
    if (glevel) ibp( 6 );      /*950504*/
    /* definelab(lab);  break/continue aus der fork-Anweisung moeglich 17.9. */
@@ -504,9 +511,9 @@ static void startstmt(lab, swp, lev) struct swtch *swp; {
      code(Gen); 
      codelist->u.node = listnodes(e, 0, 0);
    }
-   isinsyncmode = 1;
+   isinsyncmode = 1; isinsideastartbody = 1;
    statement(lab, swp, lev, 0, 0, 0);   /*CWK: no former frames to flush. */
-   isinsyncmode = 0;
+   isinsyncmode = 0; isinsideastartbody = 0;
    xproc();
 }
 
@@ -654,9 +661,9 @@ void retcode(p, lab, rframes) Tree p; {
    }
    assert( rframes >= 0 );   /*CWK*/
    p->rframes = rframes;     /*CWK*/
-   p->started = !(cfunc->sync) && isinsyncmode;     /*CWK*/
+   p->started = isinsideastartbody; /*!(cfunc->sync) && isinsyncmode;     /*CWK*/
    if (isinsyncmode && cfunc && !(cfunc->sync))              /*CWK*/
-      warning("return crossing start/join boundary will destroy synchronity\n"); 
+      warning("return crossing start/join boundary may destroy synchronity\n"); 
    walk(p, 0, 0);
    branch(lab);
 }
@@ -742,6 +749,29 @@ void statement(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; {
       expect(':');
       statement(loop, swp, lev, bframes, cframes, rframes);
       break;
+   case SEQ: {                        /*970310 CWK*/
+      extern void begseq(int), endseq(int);
+      int mylabelnr = SEQcounter++;
+      int ignore = 0;
+      if (!isFORKmodul) {
+         ignore = 1;
+         error("seq is a FORK-specific statement!\n");
+      }
+      if (!isinsyncmode) {
+         ignore = 1;
+         warning("seq in asynchronous context is ignored\n");
+      }
+      t = gettok();
+      isinsyncmode = 0;
+      walk(0,0,0);   /*damit vorangehende stmts geflusht werden 970612*/
+      if (!ignore)  begseq( mylabelnr );
+      statement(loop, swp, lev, bframes, cframes, rframes );
+      if (! ignore) {
+         endseq( mylabelnr );
+         isinsyncmode = 1;
+      }
+      break;
+      }
    case FARM: {                                                       /*CWK*/
       extern void ibp( int );
       int ignore = 0;
@@ -768,15 +798,17 @@ void statement(loop, swp, lev, bframes, cframes, rframes) struct swtch *swp; {
       if (isinsyncmode)
          error("beginsync statement is not allowed in synchronous context\n");
       t = gettok();
-      isinsyncmode = 1;
+      isinsyncmode = 1; isinsideastartbody = 1;
       statement(loop, swp, lev, 0, 0, 0);
-      isinsyncmode = 0;
+      isinsyncmode = 0; isinsideastartbody = 0;
       break;
    case RETURN:
       t = gettok();
       definept(0);
       if (cfunc->sync && !isinsyncmode)
          error("return from inside a farm statement is forbidden!\n");
+      if (!isinsyncmode && isinsideastartbody)  /*970821*/
+         warning("return crossing start/join and farm/seq boundary may destroy synchronity\n"); 
       if (t != ';')
          if (freturn(cfunc->type) == voidtype) {
             error("extraneous return value\n");
